################################################################################
# prevalidation
prevalidation <- function(
X, # data internal model
Y, # response, must be a surv-object
Z, # data external model
nperm = 500, # number of permutations for the test
method = "boosting", # boosting or LASSO
tuning = 10, # if boosting: mstop, else lambda, vectors allowed
opt.tuning = FALSE, # if boosting: mstopAIC, else lambda.cv
correction = FALSE, # correct, if no genes selected?
K=5 # number of folds used for building the predictor
)

{
# definitions/ check input
if (method == "boosting")
   { mstop <- tuning
     mstopAIC <- opt.tuning
    }
if (method == "LASSO")
   { lambda <- tuning
     lambda.cv <- opt.tuning
    }

n <- nrow(X)
X <- as.matrix(X)
centered <- t(t(X)-colMeans(X))
sigma <- 1/sd(X)
for (i in 1:dim(X)[2]) { X[,i] <- centered[,i] / sigma[i] }
# X <- t(t(X)-colMeans(X)) %*% diag(1/(sd(X))) # standardize schon hier => spter unkomplizierter!!

if (is.null(Z)) {
    Z <- matrix(ncol=0, nrow=n) # intercept in coxph not needed, in glm added automatically
   }

if (!is.Surv(Y) & !is.factor(Y)) {
        stop("Y must be a factor or a Surv object! \n") }

# predictor/ internal model
if ((n%%K)!=0) {x <- 1:(n%%K)} else {x <- c()}
partition <- as.factor(sample(c(rep(1:K, times = n%/%K), x)))

if (method=="boosting"){
    family <- c()
    if (is.Surv(Y)) {
         loss <- CoxPH()
         if (mstopAIC == TRUE ) {
              warning("AIC-optimized mstop can be applied for binary Y only. \n")
              mstopAIC <- FALSE
             }
       }
    if (is.factor(Y)) {
         loss <- Binomial()
         if (mstopAIC == TRUE ) {
              mstopaic <- c()
             }
       }
    predictor <- matrix(nrow=dim(Z)[1], ncol=ifelse(mstopAIC==TRUE,1,length(mstop)))
    for (j in 1:K){
        training.X <- X[which(partition != j), ]
        training.Y <- Y[which(partition != j)]
        test.X <- X[which(partition == j), ]
        test.Y <- Y[which(partition == j)]

        int.model <- glmboost(x=training.X, y=training.Y, family = loss,
                        control = boost_control(mstop = max(mstop), nu = 0.1, risk = "inbag"),
                        center = FALSE)
        if (mstopAIC == TRUE) {
            mstop <- mstop(AIC(int.model, method = "classical"))
            mstopaic <- c(mstopaic, mstop)
            int.model <- glmboost(x=training.X, y=training.Y, family = loss,
                        control = boost_control(mstop = mstopaic[length(mstopaic)], nu = 0.1, risk = "inbag"),
                        center = FALSE)
           }
        coefs <- matrix(nrow=dim(X)[2], ncol=length(mstop))
        for (i in length(mstop):1) {
             coefs[,i] <- coefficients(int.model[mstop[i]], which=1:dim(X)[2])
             }
        predictor[which(partition == j), ] <- test.X %*% coefs # predict(object = int.model, newdata = test.X, type="link")
       } # K

   } # boosting

if (method=="LASSO"){
    loss <- c()
    if (lambda.cv == TRUE) {
        lambda <- c()
       }
    if (is.Surv(Y)) {
        family <- "cox"
       }
    if (is.factor(Y)) {
        family <- "logistic"
       }
    predictor <- matrix(nrow=dim(Z)[1], ncol=ifelse(lambda.cv==TRUE,1,length(lambda)))
    for (j in 1:K){
        training.X <- X[which(partition != j), ]
        training.Y <- Y[which(partition != j)]
        test.X <- X[which(partition == j), ]
        test.Y <- Y[which(partition == j)]

        if (lambda.cv == TRUE) {
             suppressWarnings(rm(int.model))
             minlambda <- 1
             while(exists("int.model")==FALSE) {
                   suppressWarnings(try(int.model <- optL1(response=training.Y, penalized=training.X, unpenalized=~1, model=family,
                           lambda2=0, minlambda1 = minlambda, maxlambda1 = 20,
                           maxiter = 40, epsilon = 1e-7, trace=FALSE, standardize=FALSE, tol = .Machine$double.eps^0.1)))  # leave one out
                   minlambda <- minlambda+1
                   }
             l <- int.model$lambda
             int.model <- int.model$fullfit
             if(sum(abs(coefficients(int.model, which="penalized")))==0) { if(correction==TRUE) {
                 l <- int.model@lambda1
                 while (sum(abs(coefficients(int.model, which="penalized")))==0) {
                        l <- l - 1
                        int.model <- penalized(response = training.Y, penalized = training.X, unpenalized=~1, lambda1=l, lambda2=0, standardize=FALSE, trace = FALSE)
                        }
                } }
             lambda <- c(lambda, l)
             tuning <- lambda
             coefs <- matrix(coefficients(int.model,which="penalized"),ncol=1)
            } else {
             coefs <- matrix(nrow=dim(X)[2], ncol=length(lambda))
             for (i in 1:length(lambda)) {
             l <- lambda[i]
             int.model <- penalized(response = training.Y, penalized = training.X, lambda1=lambda[i], lambda2=0, standardize=FALSE, trace = FALSE)
             if(sum(abs(coefficients(int.model, which="penalized")))==0) { if(correction==TRUE) {
                 while (sum(abs(coefficients(int.model, which="penalized")))==0) {
                        l <- l - 1
                        int.model <- penalized(response = training.Y, penalized = training.X, unpenalized=~1, lambda1=l, lambda2=0, standardize=FALSE, trace = FALSE)
                        }
                } }
             coefs[,i] <- coefficients(int.model,which="penalized")
             lambda[i] <- l
             tuning <- lambda
             } # length lambda
             } # else => lambda.cv == FALSE
        predictor[which(partition == j), ] <- test.X %*% coefs # predict(object = int.model, newdata = test.X, type="link")
      } # K
   } # LASSO

# external model
pZ <- list()
preal <- c()
ext.models <- list()
w <- 0
for (i in 1:dim(predictor)[2]) {
if (sum(abs(predictor[,i]))>0) {
    pZ[[i]] <- cbind(predictor[,i], Z)
   } else {
    pZ[[i]] <- Z
    w <- i
    warning("The internal model results in having no genetic data at all in the external model. \n p-values for external model and permutation test were set to 1. \n")
   }

dataframe <- data.frame(Y = Y, pZ[[i]])
if (is.Surv(Y)) {
    ext.models[[i]] <- coxph(Y ~ ., data = dataframe)
    preal <- c(preal, ifelse(w!=i,summary(ext.models[[i]])$coefficients[1,5],1))
   }
if (is.factor(Y)) {
    ext.models[[i]] <- glm(Y ~ ., data = dataframe, family = binomial())
    preal <- c(preal, ifelse(w!=i,summary(ext.models[[i]])$coefficients[1,4],1))
   }
}

# permutation test
perm <- sapply(as.list(1:nperm), FUN = pv.perm, XX = X, Y = Y, Z = Z,  n = n,
        family = family, loss = loss, K = K,
        method = method, tuning=tuning, opt.tuning=opt.tuning, correction=correction)
pperm <- matrix(unlist(perm),nrow=nperm,ncol=dim(predictor)[2],byrow=TRUE)
pvalue <- c()
for (i in 1:dim(predictor)[2]) {
    pvalue <- matrix(c(pvalue, sum(pperm[ ,i] <= preal[i])/nperm), nrow = 1)
    }

if (method == "boosting") {
     tuning <- mstop
     if (mstopAIC == TRUE && is.factor(Y)==TRUE) {tuning <- mstopaic}
    } else {
     tuning <- lambda
    }

return(list(
model.external = ext.models,
predictor = predictor,
p.value.perm = pvalue,  # p value permutation test
p.value.ext =  preal, # p values external model
p.values.perm = pperm, # matrix of p values used for computing p.value.perm
tuning = tuning
))

}


################################################################################
# pv.perm
pv.perm <- function (i, XX, Y, Z, n, family, loss, K, method, tuning, opt.tuning, correction=correction)
{
if (method == "boosting")
   { mstop <- tuning
     mstopAIC <- opt.tuning
    if (is.Surv(Y)) { if (mstopAIC == TRUE ) { mstopAIC <- FALSE } }
    }
if (method == "LASSO")
   { lambda <- tuning
     lambda.cv <- opt.tuning
    }

set.seed(i)
print(paste("perm=", i, sep = ""))
sampi <- sample(n)

if ((n%%K)!=0) {x <- 1:(n%%K)} else {x <- c()}
partition <- as.factor(sample(c(rep(1:K, times = n%/%K), x)))
if (method=="boosting") {
    predictor <- matrix(nrow=dim(Z)[1], ncol=ifelse(mstopAIC==TRUE,1,length(mstop)))
    } else {
    predictor <- matrix(nrow=dim(Z)[1], ncol=ifelse(lambda.cv==TRUE,1,length(lambda)))
    }

XX <- XX[sampi, ]

for (j in 1:K){

    training.X <- XX[which(partition != j),]
    training.Y <- Y[which(partition != j)]
    test.X <- XX[which(partition == j),]
    test.Y <- Y[which(partition == j)]

    if (method=="boosting"){
    int.model <- glmboost(x=training.X, y=training.Y, family = loss, control = boost_control(mstop = max(mstop), nu = 0.1, risk = "inbag"), center = FALSE)
    if (mstopAIC == TRUE) {
        mstopaic <- mstop(AIC(int.model, method = "classical"))
        mstop <- mstopaic
        int.model <- glmboost(x=training.X, y=training.Y, family = loss,
                    control = boost_control(mstop = mstopaic, nu = 0.1, risk = "inbag"),
                    center = FALSE)
       }
    coefs <- matrix(nrow=dim(X)[2], ncol=length(mstop))
    for (i in length(mstop):1) {
         coefs[,i] <- coefficients(int.model[mstop[i]], which=1:dim(X)[2])
         }
    predictor[which(partition == j), ] <- test.X %*% coefs # predict(object = int.model, newdata = test.X, type="link")

    } # boosting

    if (method=="LASSO"){

    if (lambda.cv == TRUE) {
       suppressWarnings(rm(int.model))
       minlambda <- 1
       while(exists("int.model")==FALSE) {
             suppressWarnings(try(int.model <- optL1(response=training.Y, penalized=training.X, unpenalized=~1, model=family,
                     lambda2=0, minlambda1 = minlambda, maxlambda1 = 20,
                     maxiter = 40, epsilon = 1e-7, trace=FALSE, standardize=FALSE, tol = .Machine$double.eps^0.1)))  # leave one out
             minlambda <- minlambda+1
             }
       int.model <- int.model$fullfit
       if(sum(abs(coefficients(int.model, which="penalized")))==0) { if(correction==TRUE) {
           l <- int.model@lambda1
           while (sum(abs(coefficients(int.model, which="penalized")))==0) {
                  l <- l - 1
                  int.model <- penalized(response = training.Y, penalized = training.X, unpenalized=~1, lambda1=l, lambda2=0, standardize=FALSE, trace = FALSE)
                  }
          } }
       coefs <- matrix(coefficients(int.model,which="penalized"),ncol=1)
       } else {
       coefs <- matrix(nrow=dim(X)[2], ncol=length(lambda))
       for (i in 1:length(lambda)) {
       int.model <- penalized(response = training.Y, penalized = training.X, lambda1=lambda[i], lambda2=0, standardize=FALSE, trace = FALSE)
       if(sum(abs(coefficients(int.model, which="penalized")))==0) { if(correction==TRUE) {
           l <- int.model@lambda1
           while (sum(abs(coefficients(int.model, which="penalized")))==0) {
                  l <- l - 1
                  int.model <- penalized(response = training.Y, penalized = training.X, unpenalized=~1, lambda1=l, lambda2=0, standardize=FALSE, trace = FALSE)
                  }
          } }
       coefs[,i] <- coefficients(int.model,which="penalized")
       } # length lambda
       } # else => lambda.cv == FALSE
       predictor[which(partition == j), ] <- test.X %*% coefs # predict(object = int.model, newdata = test.X, type="link")
    } # LASSO

    }

pZ <- list()
preal <- c()
w <- 0
for (i in 1:dim(predictor)[2]) {
if (sum(abs(predictor[,i]))>0) {
    pZ[[i]] <- cbind(predictor[,i], Z)
   } else {
    pZ[[i]] <- Z
    w <- i
   }

dataframe <- data.frame(Y = Y, pZ[[i]])
if (is.Surv(Y)) {
    ext.model <- coxph(Y ~ ., data = dataframe)
    preal <- c(preal, ifelse(w!=i,summary(ext.model)$coefficients[1,5],1))
   }
if (is.factor(Y)) {
    ext.model <- glm(Y ~ ., data = dataframe, family = binomial())
    preal <- c(preal, ifelse(w!=i,summary(ext.model)$coefficients[1,4],1))
   }
}


return(list(pperm=preal))
}


################################################################################
# plot.prevalidation
plot.prevalidation <- function(
pv,
main = ""
)

{
# check input
names.output <- c("model.external", "predictor", "p.value.perm", "p.value.ext", "p.values.perm", "tuning")
for(j in names.output)
if ((j %in% rownames(summary(pv)))==FALSE) {
    stop("Input must be output of function prevalidation. \n")
    }

# definitions
tuning <- pv$tuning
p.value.perm <- pv$p.value.perm
p.values.perm <- pv$p.values.perm
nperm <- dim(p.values.perm)[1]

# plot
par(las=3)
plot(tuning, p.value.perm, type = "n",
    ylim = c(0,1),
    main = main,
    axes = FALSE,
    xlab = "tuning", ylab = "p-value", col = 1)
for (i in 1:nperm) {
    points(tuning, p.values.perm[i, ], type = "l", lty = 3, col = 8)
}
points(tuning, p.value.perm, type = "l", col = 1)

# the good look
box()
axis(1, at=tuning, labels=paste(tuning),
        col = "black", lty = 1, lwd = 1 )
axis(2, at=c(0, 0.2, 0.4, 0.6, 0.8, 1), labels=c("0.0", "0.2", "0.4", "0.6", "0.8", "1.0" ),
        col = "black", lty = 1, lwd = 1 )

}


################################################################################
# globalboosttest.plot
globalboostest.plot <- function (gbtobject) {         # plot funktion
   mstop <- dim(gbtobject$riskperm)[1]
   nperm <- dim(gbtobject$riskperm)[2]
   plot(1:max(mstop), gbtobject$riskreal, type = "n", ylim = c(min(gbtobject$riskreal,
            gbtobject$riskperm), max(gbtobject$riskreal, gbtobject$riskperm)),# main = "Risk",
            xlab = "mstop", ylab = "Risk", col = 1)
   for (i in 1:nperm) {
            points(1:max(mstop), gbtobject$riskperm[, i], type = "l", lty = 3,
                col = 8)
        }
   points(1:max(mstop), gbtobject$riskreal, type = "l", col = 1)

}


################################################################################
# simulation
simulation <- function (
fam = "logistic", # family: "cox" or "logistic" (with natural link function only)
alpha = 1, # scale parameter weibull family
baseline = 1, # baseline hazard rate for cox model of survival times
n = 200, # number observations
n.clinic = 5, # number of clinical covariates
coef.clinic = c(0, .5, 2, 1.5, 1), # coefficient vector clinical covariates
n.micro = 10, # number of micro array covariates
n.infl = 2, # number of genes having impact on response y
coef.micro = c(0,1), # parameters for uniform distribution of coefficients for mirco-array
correlation = NULL, # matrix of dimension n.clinic x (d < n.micro) with correlations between clinical covariates and d genes
n.infl.corr = 0, # number of genes with impact and correlation
censoring = 0.1 # hazard rate for censoring (same distriubtion as for survival times, they only differ in the hazard rate)
)
# all covariates are normal, centered and standardized: N(0,1)
# clincial coefficients are specified,
# coefficients of micro-array are uniform distributed with parameters given in coef.micro
# return: response, clincal data, genes, a matrix:
# row 1: coefficient vector of  genes
# row 2 indicates which genes are correlated with the clincial covariates, the entry (between 1 and d) indicates which correlation (see input argument correlation) is ment
{
# check input
check.simulation(fam = fam, alpha = alpha, n = n,
n.clinic = n.clinic, coef.clinic = coef.clinic,
n.micro = n.micro, n.infl = n.infl, coef.micro = coef.micro,
correlation = correlation, n.infl.corr = n.infl.corr, censoring = censoring)

# definitions
if(is.null(correlation)==FALSE){ d <- dim(correlation)[2] } else { d <- 0 }

# covariance matrix + clinical data
if (n.clinic + d > 0 ) {
    covariance <- diag(n.clinic + d)
    if (d>0) { covariance[0:n.clinic, (n.clinic+1):(n.clinic+d)] <- correlation }
    covariance <- forceSymmetric(covariance)
    if (sum(as.numeric(eigen(covariance)[[1]]>0))== dim(covariance)[1]) { # covariance positive definite
        z <- mvrnorm(n=n, mu=rep(0,n.clinic+d), Sigma=covariance, tol = 1e-6, empirical = FALSE)
        } else { # covariance not positive definite, but with clear structure
        z <- alternative.correlation(correlation, n, n.clinic)
        }
    Z <- z[ ,1:n.clinic]
    } else {
    Z <- matrix(nrow=n,ncol=0)
    }

# micro array
if (n.micro > 0){
    X <- matrix(nrow=n, ncol=n.micro)
    if (d>0) {X[ ,1:d] <- z[ ,(n.clinic+1):(n.clinic+d)]}
    if ((d+1)<(n.micro)) {for (i in (d+1):(n.micro)) { X[,i] <- rnorm(n,0,1) }}
    mix <- sample(n.micro)
    X <- X[ , mix]
    # coefficients
    infl <- runif(n.infl, coef.micro[1], coef.micro[2] )
    coef.micro <- rep(0,n.micro)
    which.correlated <- which(mix < d+1)
    which.not.correlated <- which(mix > d)
    where.which.corr <- mix[which.correlated]
    if (d>0) {which.correlated.infl <- sample(which.correlated, n.infl.corr)} else {which.correlated.infl <- 0}
    if (d<n.micro) {which.not.correlated.infl <- sample(which.not.correlated, (n.infl-n.infl.corr))} else {which.not.correlated.infl <- 0}
    coef.micro[c(which.correlated.infl, which.not.correlated.infl)] <- infl
    } else {
    X <- matrix(nrow=n,ncol=0)
    }

# predictor
eta <- cbind(Z,X)%*% c(coef.clinic, coef.micro)

# response
if (fam == "cox") {
    hazard <- exp(eta)*baseline
    verteilung <- function(zeit, hazard){1-exp(-(hazard*zeit/alpha))}
    verteilung.inverse <- function(vtlg, hazard){-log(1-vtlg)*alpha/hazard}
    zeit.ereignis <- round(verteilung.inverse(runif(n,0,1), hazard), digits=2)
    zeit.zensierung <- round(verteilung.inverse(runif(n,0,1), rep(censoring,n)), digits=2)
    zensierung <- as.numeric(zeit.ereignis < zeit.zensierung)
    zeit <-  zensierung*zeit.ereignis + (1-zensierung)*zeit.zensierung
    Y <- Surv(time=zeit, event=zensierung, type=c('right') )
   }
if (fam == "logistic") {
    fam <- binomial()
    link <- fam$linkinv
    E <- as.vector(link(eta))
    Y <- rbinom(n, 1, E)
   }

# return
correlation.ind <- rep(0, n.micro)
correlation.ind[which.correlated] <- mix[which.correlated]
info <- matrix(c(coef.micro, correlation.ind), ncol=n.micro, nrow=2, byrow=TRUE)
return(list(
X=X, Y=Y, Z=Z, info = info
))

}


################################################################################
# check.simulation
check.simulation <- function(
fam = "logistic", # family: "cox" or "logistic"
alpha = 1, # scale weibull
n = 200, # number observations
n.clinic = 5, # number of clinical covariates
coef.clinic = c(0, .5, 2, 1.5, 1), # coefficient vector clinical covariates
n.micro = 10, # number of micro array covariates
n.infl = 2, # number of genes having impact on response y
coef.micro = c(0,1), # parameters for uniform distribution of coefficients for
                     # mirco-array
correlation = NULL, # matrix of dimension n.clinic x (d < n.micro) with
                    # correlations between clinical and the first d genes
n.infl.corr = 0, # number of genes with impact and correlation
censoring = 0.4
)
{
is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol

# fam
    if ((fam %in%c("cox", "logistic"))==FALSE)
         stop ("fam must be 'cox' or 'logistic'. \n")

# alpha
    if ( length(alpha)!=1 && fam =="cox" )
         stop ("alpha must be a sinlge number > 0. \n")
    if ( alpha<0 && fam =="cox" )
         stop ("alpha must be a sinlge number > 0. \n")

# n
    if ( length(n)!=1)
         stop ("n must be a sinlge integer > 0. \n")
    if ( n<0)
         stop ("n must be a sinlge integer > 0. \n")
    if ( is.wholenumber(n)==FALSE)
         stop ("n must be a sinlge integer > 0. \n")

# n.clinic
    if ( length(n.clinic)!=1)
         stop ("n.clinic must be a sinlge integer >= 0. \n")
    if ( n.clinic<0)
         stop ("n.clinic must be a sinlge integer >= 0. \n")
    if ( is.wholenumber(n.clinic)==FALSE)
         stop ("n.clinic must be a sinlge integer >= 0. \n")

# coef.clinic
    if ( is.vector(coef.clinic)==FALSE)
         stop ("coef.clinic must be a vector of length n.clinic. \n")
    if ( length(coef.clinic)!=n.clinic)
         stop ("coef.clinic must be a vector of length n.clinic. \n")

# n.micro
    if ( length(n.micro)!=1)
         stop ("n.micro must be a sinlge integer >= 0. \n")
    if ( n.micro<0)
         stop ("n.micro must be a sinlge integer >= 0. \n")
    if ( is.wholenumber(n.micro)==FALSE)
         stop ("n.micro must be a sinlge integer >= 0. \n")

# n.infl
    if ( length(n.infl)!=1)
         stop ("n.v must be a sinlge integer > 0. \n")
    if ( n.infl<0)
         stop ("n.infl must be a sinlge integer > 0. \n")
    if ( n.infl>n.micro)
         stop ("n.infl must be a sinlge integer < n.micro. \n")
    if ( is.wholenumber(n.infl)==FALSE)
         stop ("n.infl must be a sinlge integer > 0. \n")

# coef.micro
    if ( is.vector(coef.micro)==FALSE)
         stop ("coef.micro must be a vector of length 2. \n")
    if ( length(coef.micro)!=2)
         stop ("coef.micro must be a vector of length 2. \n")

# correlation
    if ( is.matrix(correlation)==TRUE ) {
         if (dim(correlation)[1]!=n.clinic) {
             stop ("'correlation' must have n.clinic rows. \n")
             }
         if (dim(correlation)[1]>n.micro) {
             stop ("'correlation' must have less coloums as n.micro. \n")
             }
         if ( max(correlation)>= 1 ) {
             stop ("'correlation' must contain correlations. \n")
             }
         if ( min(correlation)<= -1 ) {
             stop ("'correlation' must contain correlations. \n")
             }
         }
        if ( is.null(correlation)==FALSE && is.matrix(correlation)==FALSE) {
             stop ("'correlation' must be either NULL or a matrix. \n")
            }

# n.infl.corr
    if ( length(n.infl.corr)!=1)
         stop ("n.infl.corr must be a sinlge integer >= 0. \n")
    if ( n.infl.corr<0)
         stop ("n.infl.corr must be a sinlge integer >= 0. \n")
    if ( is.wholenumber(n.infl.corr)==FALSE)
         stop ("n.infl.corr must be a sinlge integer >= 0. \n")
    if ( is.null(correlation)==TRUE ) {
         if ( n.infl.corr>0) {
              stop ("There must be correlated genes if n.infl.corr is > 0. \n")
            } } else {
         if ( n.infl.corr>length(which(colSums(abs(correlation))>0)) ) {
              stop ("There must be more correlated genes as n.infl.corr. \n")}
         if ( length(which(colSums(abs(correlation))>0)) + (n.infl-n.infl.corr) > n.micro )  {
              stop ("Number of correlated and uncorrelated coefficients must be consistent with the number of overall correlated variables. \n")}
       }
    if ( n.infl.corr>n.infl)
         stop ("n.infl must be larger as n.infl.corr. \n")

# censoring
    if ( length(censoring)!=1)
         stop ("censoring must be a sinlge number > 0. \n")
    if ( censoring<0)
         stop ("censoring must be a sinlge number > 0. \n")
    if ( censoring>1)
         stop ("censoring must be a sinlge number < 1. \n")

}


################################################################################
# alternative.correlation
alternative.correlation <- function(correlation, n, n.clinic)
{
# check structure
  nn <- 0
  for (i in 1:dim(correlation)[2]) { if (length(which(correlation[,i]!=0))==1) {nn <- nn+1} }
  if (nn != dim(correlation)[2]) { stop("Implied covariance matrix is not positive definite and has a too complicated structure for alternatives. \n")}

# simulate data
  z <- mvrnorm(n=n, mu=rep(0,n.clinic), Sigma=diag(n.clinic), tol = 1e-6, empirical = FALSE)
  gene <- matrix(nrow=n, ncol=0)
  for (i in 1:dim(correlation)[2]) {
       clinic.i <- which(correlation[,i]!=0)
       corr.i <- matrix(correlation[clinic.i,i],2,2)
                 # correlation between 1 clinical covariate and 1 gene
       diag(corr.i) <- 1
       chol.corr.i <- t(chol(corr.i))
       # cholesky decomposition of correlation between 1 clinical covariate and 1 gene, \Sigma^{-1/2}
       z1 <- z[,clinic.i]
       y2 <- rnorm(n,0,1)
       y1 <- (z1 - chol.corr.i[1,2]*y2) / chol.corr.i[1,1]
       z2 <- chol.corr.i[2,1]*y1 + chol.corr.i[2,2]*y2
       gene <- cbind(gene, z2)
  }

# return z
  z <- cbind(z, gene)

}

