# Figure 1: Setting V

# paths
p <-
setwd(p)

# libraries / functions
library(survival)
library(globalboosttest)
library(penalized)
library(Matrix)
library(MASS)
source("functions.R")

# model parameters
n <- 200
n.clinic <- 5
n.micro <- 1000
coef.micro <- c(0.1,0.7)
n.infl <- 20

correlation <- function(rho) { m <- matrix(0, nrow=5, ncol=50)
    for (j in 1:5) { m[j,((j*10)-9):(j*10)] <- rho }
    return(m) }
corr0 <- NULL
corr2 <- correlation(0.2)
corr8 <- correlation(0.8)
corrl <- list(corr0 = corr0, corr2 = corr2, corr8=corr8)

n.infl.corr <- c(0,5,10,20)

# cox model
alpha <- 1
baseline <- .1
censoring <- .1
coef.clinic.c <- c(0, .5, 2, -1.5, -1)

# simulation parameters
seed <- 1234
set.seed(seed)
n.rep <- 100

stops <-  c(50,100,200,500,1000)
lambda <- c(5,10,15,20,25)
permutationen <-  1000

# indices => it is setting V
i <- 2  # corrl
j <- 3  # n.infl.corr


# data
liste <- c()
for (l in 1:n.rep) {
data <- simulation(fam = "cox", alpha = alpha, baseline=baseline, n = n, n.clinic = n.clinic,
        coef.clinic = coef.clinic.c, n.micro = n.micro, n.infl = n.infl,
        coef.micro = coef.micro, correlation = corrl[[i]], n.infl.corr = n.infl.corr[j],
        censoring = censoring)
assign(paste("d.c", names(corrl)[i], as.character(n.infl.corr)[j], l, sep="."), data)
liste <- c(liste, paste("d.c", names(corrl)[i], as.character(n.infl.corr)[j], l, sep="."))
}

save(file=paste(p, "settingV.data", ".RData", sep=""), list=liste)
load(file=paste(p, "settingV.data", ".RData", sep=""))

# gbt
  p.gbt <- matrix(ncol=length(stops), nrow=100)
  colnames(p.gbt) <- as.character(stops)
  liste <- c()
  for (l in 1:n.rep)  {
  d <- paste("d.c", names(corrl)[i], as.character(n.infl.corr)[j], l, sep=".")
  X <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$X", sep="")))
  Y <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$Y", sep="")))
  Z <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$Z", sep="")))
  gbt <- globalboosttest(X=X, Y=Y, Z=Z, nperm=permutationen, mstop=stops, plot=FALSE, pvalueonly=F)
  assign(paste(d, "gbt", l, sep="."), gbt)
  liste <- c(liste, paste(d, "gbt", l, sep="."))
  p.gbt[l,] <- eval(parse(text= paste(d, ".gbt.", l, "$pvalue", sep="")))
  }

# pv
  method <- "boosting"
  p.pv.b <- matrix(ncol=length(stops), nrow=n.rep)
  colnames(p.pv.b) <- as.character(stops)
  liste <- c()
  for (l in 1:n.rep)  {
  d <- paste("d.c", names(corrl)[i], as.character(n.infl.corr)[j], l, sep=".")
  X <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$X", sep="")))
  Y <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$Y", sep="")))
  Z <- eval(parse(text=paste("d.c.", names(corrl)[i], ".", as.character(n.infl.corr)[j], ".", l, "$Z", sep="")))
  pv.b <- prevalidation(X, Y, Z, nperm = permutationen, method = method, tuning = stops)
  assign(paste(d, "pv.b", l, sep="."), pv.b)
  liste <- c(liste, paste(d, "pv.b", l, sep="."))
  p.pv.b[l,] <- eval(parse(text= paste(d, ".pv.b.", l, "$p.value.perm", sep="")))
  }

save(file=paste(p,"settingV.results", ".RData",sep=""), list=c("p.gbt", "p.pv.b", "i", "j"))


