# Figure 2: Setting XV -- perfect correlation
e <- "extra5"

# paths
p <-
setwd(p)

# libraries / functions
library(survival)
library(globalboosttest)
library(penalized)
library(Matrix)
library(MASS)
source("functions.R")

# model parameters
n <- 200
n.clinic <- 5
n.micro <- 1000
coef.micro <- c(0.1,0.7)
n.infl <- 20

correlation <- function(rho) { m <- matrix(0, nrow=5, ncol=50)
    for (j in 1:5) { m[j,((j*10)-9):(j*10)] <- rho }
    return(m) }
corr1 <- correlation(0.9999)
corrl <- list(corr1 = corr1)

n.infl.corr <- c(0,5,10,20)

# cox model
alpha <- 1
baseline <- .1
censoring <- .1
coef.clinic.c <- c(0, .5, 2, -1.5, -1)

# simulation parameters
seed <- 1234
set.seed(seed)
n.rep <- 100

stops <-  c(50,100,200,500,1000)
permutationen <-  1000


# indices
i <- 1  # corrl
j <- 4  # n.infl.corr


# generate data
liste <- c()
for (l in 1:n.rep) {
data <- simulation(fam = "cox", alpha = alpha, baseline=baseline, n = n, n.clinic = n.clinic,
        coef.clinic = coef.clinic.c, n.micro = n.micro, n.infl = n.infl,
        coef.micro = coef.micro, correlation = corrl[[i]], n.infl.corr = n.infl.corr[j],
        censoring = censoring)
assign(paste(e, "d", l, sep="."), data)
liste <- c(liste, paste(e, "d", l, sep="."))
}

save(file=paste(p, "settingXV.data", ".RData", sep=""), list=liste)
load(paste(p, "settingXV.data", ".RData", sep="") )

# gbt
  p.gbt <- matrix(ncol=length(stops), nrow=100)
  colnames(p.gbt) <- as.character(stops)
  liste <- c()
  for (l in 1:n.rep)  {
  d <- paste(e, "d", l, sep=".")
  X <- eval(parse(text=paste(e, ".d", ".", l, "$X", sep="")))
  Y <- eval(parse(text=paste(e, ".d", ".", l, "$Y", sep="")))
  Z <- eval(parse(text=paste(e, ".d", ".", l, "$Z", sep="")))
  gbt <- globalboosttest(X=X, Y=Y, Z=Z, nperm=permutationen, mstop=stops, plot=FALSE, pvalueonly=F)
  assign(paste(d, "gbt", l, sep="."), gbt)
  liste <- c(liste, paste(d, "gbt", l, sep="."))
  p.gbt[l,] <- eval(parse(text= paste(d, ".gbt.", l, "$pvalue", sep="")))
  }

# pv
  method <- "boosting"
  liste <- c()
  p.pv.b <- matrix(ncol=length(stops), nrow=100)
  colnames(p.pv.b) <- as.character(stops)
  for (l in 1:n.rep)  {
  d <- paste(e, "d", l, sep=".")
  X <- eval(parse(text=paste(e, ".d", ".", l, "$X", sep="")))
  Y <- eval(parse(text=paste(e, ".d", ".", l, "$Y", sep="")))
  Z <- eval(parse(text=paste(e, ".d", ".", l, "$Z", sep="")))
  pv.b <- prevalidation(X, Y, Z, nperm = permutationen, method = method, tuning = stops)
  assign(paste(d, "pv.b", l, sep="."), pv.b)
  liste <- c(liste, paste(d, "pv.b", l, sep="."))
  p.pv.b[l,] <- eval(parse(text= paste(d, ".pv.b.", l, "$p.value.perm", sep="")))
  }

# save results
save(file=paste(p, "settingXV.results", ".RData",sep=""),
list=c("i", "j", "p.gbt", "p.pv.b"))

